@@ -366,26 +366,14 @@ gfu_common_device_flags_to_string(guint64 device_flag)
 		/* skip */
 		return NULL;
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_NO_AUTO_INSTANCE_IDS) {
-		/* skip */
-		return NULL;
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION) {
 		/* TRANSLATORS: Device update needs to be separately activated */
 		return _("Device update needs activation");
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_ENSURE_SEMVER) {
-		/* skip */
-		return NULL;
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_HISTORICAL) {
 		/* skip */
 		return NULL;
 	}
-	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_SUPPORTED) {
-		/* skip */
-		return NULL;
-	}
 	if (device_flag == FWUPD_DEVICE_FLAG_WILL_DISAPPEAR) {
 		/* TRANSLATORS: Device will not return after update completes */
 		return _("Device will not re-appear after update completes");
@@ -516,12 +504,8 @@ gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 		return "dialog-error-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED)
 		return "media-floppy-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_NO_AUTO_INSTANCE_IDS)
-		return "dialog-error-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_ACTIVATION)
 		return "emblem-important-symbolic";
-	if (device_flag == FWUPD_DEVICE_FLAG_ENSURE_SEMVER)
-		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_WILL_DISAPPEAR)
 		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_CAN_VERIFY)
@@ -623,6 +607,12 @@ gfu_status_to_string(FwupdStatus status)
 		/* TRANSLATORS: waiting for daemon */
 		return _("Shutting down…");
 		break;
+#if FWUPD_CHECK_VERSION(1, 9, 8)
+	case FWUPD_STATUS_WAITING_FOR_USER:
+		/* TRANSLATORS: waiting for daemon */
+		return _("Waiting for user…");
+		break;
+#endif
 	default:
 		break;
 	}
