@@ -17,6 +17,8 @@
 #include "gfu-device-row.h"
 #include "gfu-release-row.h"
 
+typedef void (*GfuMainDialogFunc)(gpointer user_data);
+
 typedef struct {
 	AdwApplication *application;
 	GtkBuilder *builder;
@@ -26,7 +28,9 @@ typedef struct {
 	FwupdRelease *release;
 	FwupdInstallFlags flags;
 	guint refresh_id;
-	GPtrArray *actions;
+	GPtrArray *dialogs;	   /* of GtkDialog */
+	GtkWindow *dialog_current; /* no-ref */
+	GfuMainDialogFunc dialog_func;
 	GPtrArray *post_requests;
 } GfuMain;
 
@@ -34,6 +38,8 @@ static void
 gfu_main_release_row_button_clicked_cb(GfuReleaseRow *row, GfuMain *self);
 static void
 gfu_main_refresh_releases(GfuMain *self);
+static void
+gfu_main_process_dialogs(GfuMain *self);
 
 /* GTK helper functions */
 
@@ -45,6 +51,15 @@ gfu_main_list_box_remove_all(GtkListBox *container)
 		gtk_list_box_remove(container, child);
 }
 
+/* this takes ownership */
+static void
+gfu_main_add_dialog(GfuMain *self, GtkWindow *dialog, gboolean process_now)
+{
+	g_ptr_array_add(self->dialogs, dialog);
+	if (process_now)
+		gfu_main_process_dialogs(self);
+}
+
 static void
 gfu_main_error_dialog(GfuMain *self, const gchar *title, const gchar *message)
 {
@@ -60,8 +75,7 @@ gfu_main_error_dialog(GfuMain *self, const gchar *title, const gchar *message)
 					title);
 	if (message != NULL)
 		gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", message);
-	g_signal_connect(dialog, "response", G_CALLBACK(gtk_window_destroy), NULL);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 }
 
 static void
@@ -302,6 +316,12 @@ gfu_main_device_problem_to_string(GfuMain *self, FwupdDeviceProblem problem)
 		/* TRANSLATORS: device cannot be interrupted, for instance taking a phone call */
 		return g_strdup(_("Device is in use"));
 	}
+#endif
+#if FWUPD_CHECK_VERSION(1, 9, 6)
+	if (problem == FWUPD_DEVICE_PROBLEM_DISPLAY_REQUIRED) {
+		/* TRANSLATORS: device cannot be interrupted, for instance taking a phone call */
+		return g_strdup(_("Device cannot be used while there are no displays plugged in"));
+	}
 #endif
 	return NULL;
 }
@@ -471,9 +491,7 @@ gfu_main_device_added_cb(FwupdClient *client, FwupdDevice *device, GfuMain *self
 	GtkWidget *w;
 	GtkWidget *l;
 
-	/* ignore if device can't be updated */
-	if (!fwupd_device_has_flag(device, FWUPD_DEVICE_FLAG_UPDATABLE) &&
-	    !fwupd_device_has_flag(device, FWUPD_DEVICE_FLAG_UPDATABLE_HIDDEN))
+	if (fwupd_device_get_version(device) == NULL)
 		return;
 
 	/* create and add new row for device */
@@ -512,7 +530,6 @@ gfu_main_reboot_shutdown_response_cb(GtkDialog *dialog, int response_id, GfuMain
 					      _("A manual shutdown is required."));
 		}
 	}
-	gtk_window_destroy(GTK_WINDOW(dialog));
 }
 
 static void
@@ -528,7 +545,6 @@ gfu_main_reboot_restart_response_cb(GtkDialog *dialog, int response_id, GfuMain
 					      _("A manual reboot is required."));
 		}
 	}
-	gtk_window_destroy(GTK_WINDOW(dialog));
 }
 
 static void
@@ -549,11 +565,25 @@ gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	}
 	for (guint i = 0; i < remotes->len; i++) {
 		FwupdRemote *remote = g_ptr_array_index(remotes, i);
+#if FWUPD_CHECK_VERSION(2, 0, 0)
+		/* if another repository is turned on providing metadata */
+		g_debug("%s is %s",
+			fwupd_remote_get_id(remote),
+			fwupd_remote_has_flag(remote, FWUPD_REMOTE_FLAG_ENABLED) ? "enabled"
+										 : "disabled");
+		if (fwupd_remote_has_flag(remote, FWUPD_REMOTE_FLAG_ENABLED)) {
+			if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_DOWNLOAD)
+				enabled_any_download_remote = TRUE;
+			/* lvfs is present and disabled */
+		} else {
+			if (g_strcmp0(fwupd_remote_get_id(remote), "lvfs") == 0)
+				disabled_lvfs_remote = TRUE;
+		}
+#else
+		/* if another repository is turned on providing metadata */
 		g_debug("%s is %s",
 			fwupd_remote_get_id(remote),
 			fwupd_remote_get_enabled(remote) ? "enabled" : "disabled");
-
-		/* if another repository is turned on providing metadata */
 		if (fwupd_remote_get_enabled(remote)) {
 			if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_DOWNLOAD)
 				enabled_any_download_remote = TRUE;
@@ -563,6 +593,7 @@ gfu_main_update_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 			if (g_strcmp0(fwupd_remote_get_id(remote), "lvfs") == 0)
 				disabled_lvfs_remote = TRUE;
 		}
+#endif
 	}
 
 	w = GTK_WIDGET(gtk_builder_get_object(self->builder, "infobar_enable_lvfs"));
@@ -651,13 +682,9 @@ gfu_main_update_devices_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 		FwupdDevice *device = g_ptr_array_index(devices, i);
 		GtkWidget *l;
 
-		/* skip devices that can't be updated */
-		if (!fwupd_device_has_flag(device, FWUPD_DEVICE_FLAG_UPDATABLE) &&
-		    !fwupd_device_has_flag(device, FWUPD_DEVICE_FLAG_UPDATABLE_HIDDEN) &&
-		    !fwupd_device_has_flag(device, FWUPD_DEVICE_FLAG_LOCKED)) {
-			g_debug("ignoring non-updatable device: %s", fwupd_device_get_name(device));
+		/* skip devices that don't have a firmware version */
+		if (fwupd_device_get_version(device) == NULL)
 			continue;
-		}
 		l = gfu_device_row_new(device);
 		gtk_widget_set_visible(l, TRUE);
 		gtk_list_box_insert(GTK_LIST_BOX(w), l, -1);
@@ -787,11 +814,20 @@ gfu_main_get_lvfs_remote_cb(GObject *source, GAsyncResult *res, gpointer user_da
 		gfu_main_error_dialog(self, _("Failed to find LVFS"), error->message);
 		return;
 	}
+#if FWUPD_CHECK_VERSION(2, 0, 0)
+	fwupd_client_refresh_remote_async(self->client,
+					  remote,
+					  FWUPD_CLIENT_DOWNLOAD_FLAG_NONE,
+					  self->cancellable,
+					  gfu_main_refresh_remote_cb,
+					  self);
+#else
 	fwupd_client_refresh_remote_async(self->client,
 					  remote,
 					  self->cancellable,
 					  gfu_main_refresh_remote_cb,
 					  self);
+#endif
 }
 
 static void
@@ -848,6 +884,19 @@ gfu_main_get_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 	}
 	for (guint i = 0; i < remotes->len; i++) {
 		FwupdRemote *remote = g_ptr_array_index(remotes, i);
+#if FWUPD_CHECK_VERSION(2, 0, 0)
+		if (!fwupd_remote_has_flag(remote, FWUPD_REMOTE_FLAG_ENABLED))
+			continue;
+		if (fwupd_remote_get_kind(remote) != FWUPD_REMOTE_KIND_DOWNLOAD)
+			continue;
+		/* should this be a single async action? */
+		fwupd_client_refresh_remote_async(self->client,
+						  remote,
+						  FWUPD_CLIENT_DOWNLOAD_FLAG_NONE,
+						  self->cancellable,
+						  gfu_main_refresh_remote_cb,
+						  self);
+#else
 		if (!fwupd_remote_get_enabled(remote))
 			continue;
 		if (fwupd_remote_get_kind(remote) != FWUPD_REMOTE_KIND_DOWNLOAD)
@@ -858,6 +907,7 @@ gfu_main_get_remotes_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 						  self->cancellable,
 						  gfu_main_refresh_remote_cb,
 						  self);
+#endif
 	}
 }
 
@@ -907,8 +957,7 @@ gfu_main_show_request_real(GfuMain *self, FwupdRequest *request, GdkPixbuf *pixb
 		gtk_box_append(GTK_BOX(box), image);
 	}
 
-	g_signal_connect(dialog, "response", G_CALLBACK(gtk_window_destroy), NULL);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 }
 
 typedef struct {
@@ -1016,7 +1065,7 @@ gfu_main_install_release_cb(GObject *source, GAsyncResult *res, gpointer user_da
 				 "response",
 				 G_CALLBACK(gfu_main_reboot_shutdown_response_cb),
 				 self);
-		gtk_window_present(GTK_WINDOW(dialog));
+		gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	} else if (fwupd_device_has_flag(self->device, FWUPD_DEVICE_FLAG_NEEDS_REBOOT)) {
 		dialog = gtk_message_dialog_new(window,
 						GTK_DIALOG_MODAL,
@@ -1032,7 +1081,7 @@ gfu_main_install_release_cb(GObject *source, GAsyncResult *res, gpointer user_da
 				 "response",
 				 G_CALLBACK(gfu_main_reboot_restart_response_cb),
 				 self);
-		gtk_window_present(GTK_WINDOW(dialog));
+		gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	} else if (fwupd_release_get_update_message(self->release) == NULL) {
 		dialog = gtk_message_dialog_new(window,
 						GTK_DIALOG_MODAL,
@@ -1048,15 +1097,26 @@ gfu_main_install_release_cb(GObject *source, GAsyncResult *res, gpointer user_da
 							 _("Installed firmware version %s on %s."),
 							 fwupd_release_get_version(self->release),
 							 fwupd_device_get_name(self->device));
-		g_signal_connect(dialog, "response", G_CALLBACK(gtk_window_destroy), NULL);
-		gtk_window_present(GTK_WINDOW(dialog));
+		gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	}
 }
 
 static void
-gfu_main_release_install(GfuMain *self)
+gfu_main_release_install_cb(gpointer user_data)
 {
+	GfuMain *self = (GfuMain *)user_data;
+
 	/* begin installing, show loading animation */
+#if FWUPD_CHECK_VERSION(2, 0, 0)
+	fwupd_client_install_release_async(self->client,
+					   self->device,
+					   self->release,
+					   self->flags | FWUPD_INSTALL_FLAG_ALLOW_BRANCH_SWITCH,
+					   FWUPD_CLIENT_DOWNLOAD_FLAG_NONE,
+					   self->cancellable,
+					   gfu_main_install_release_cb,
+					   self);
+#else
 	fwupd_client_install_release2_async(self->client,
 					    self->device,
 					    self->release,
@@ -1065,9 +1125,10 @@ gfu_main_release_install(GfuMain *self)
 					    self->cancellable,
 					    gfu_main_install_release_cb,
 					    self);
+#endif
 }
 
-static GtkWidget *
+static GtkWindow *
 gfu_main_release_show_fde_warning(GfuMain *self)
 {
 	GtkWidget *window;
@@ -1093,10 +1154,10 @@ gfu_main_release_show_fde_warning(GfuMain *self)
 	    _("Please ensure you have the volume recovery key before continuing."));
 
 	/* success */
-	return dialog;
+	return GTK_WINDOW(dialog);
 }
 
-static GtkWidget *
+static GtkWindow *
 gfu_main_release_show_branch_warning(GfuMain *self)
 {
 	GtkWidget *dialog;
@@ -1151,10 +1212,10 @@ gfu_main_release_show_branch_warning(GfuMain *self)
 	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", body->str);
 
 	/* success */
-	return dialog;
+	return GTK_WINDOW(dialog);
 }
 
-static GtkWidget *
+static GtkWindow *
 gfu_main_release_show_confirmation(GfuMain *self)
 {
 	GtkWidget *window;
@@ -1167,16 +1228,16 @@ gfu_main_release_show_confirmation(GfuMain *self)
 	/* make sure user wants to install file to device */
 	if (reinstall) {
 		/* TRANSLATORS: %1 is device name, %2 is the version */
-		title_string = _("Reinstall %s firmware version %s");
+		title_string = _("Reinstall %s firmware version %s?");
 	} else if (upgrade) {
 		/* TRANSLATORS: %1 is device name, %2 is the version */
-		title_string = _("Upgrade %s firmware version %s");
+		title_string = _("Upgrade %s firmware version %s?");
 	} else if (downgrade) {
 		/* TRANSLATORS: %1 is device name, %2 is the version */
-		title_string = _("Downgrade %s firmware version %s");
+		title_string = _("Downgrade %s firmware version %s?");
 	} else {
 		/* TRANSLATORS: %1 is device name, %2 is the version */
-		title_string = _("Install %s firmware version %s");
+		title_string = _("Install %s firmware version %s?");
 	}
 	window = GTK_WIDGET(gtk_builder_get_object(self->builder, "dialog_main"));
 	dialog = gtk_message_dialog_new(GTK_WINDOW(window),
@@ -1187,22 +1248,22 @@ gfu_main_release_show_confirmation(GfuMain *self)
 					fwupd_device_get_name(self->device),
 					fwupd_release_get_version(self->release));
 	/* TRANSLATORS: button text */
-	gtk_dialog_add_button(GTK_DIALOG(dialog), _("Cancel"), GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
 
 	self->flags = FWUPD_INSTALL_FLAG_NONE;
 	if (reinstall) {
 		/* TRANSLATORS: button text: install the same version again */
-		gtk_dialog_add_button(GTK_DIALOG(dialog), _("Reinstall"), GTK_RESPONSE_OK);
+		gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Reinstall"), GTK_RESPONSE_OK);
 		self->flags |= FWUPD_INSTALL_FLAG_ALLOW_REINSTALL;
 		/* upgrade or downgrade */
 	} else {
 		if (upgrade) {
 			/* TRANSLATORS: button text, move from old -> new */
-			gtk_dialog_add_button(GTK_DIALOG(dialog), _("Upgrade"), GTK_RESPONSE_OK);
+			gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Upgrade"), GTK_RESPONSE_OK);
 		} else if (downgrade) {
 			self->flags |= FWUPD_INSTALL_FLAG_ALLOW_OLDER;
 			/* TRANSLATORS: button text, move from new -> old */
-			gtk_dialog_add_button(GTK_DIALOG(dialog), _("Downgrade"), GTK_RESPONSE_OK);
+			gtk_dialog_add_button(GTK_DIALOG(dialog), _("_Downgrade"), GTK_RESPONSE_OK);
 		}
 	}
 	if (fwupd_device_has_flag(self->device, FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE)) {
@@ -1220,41 +1281,42 @@ gfu_main_release_show_confirmation(GfuMain *self)
 	}
 
 	/* success */
-	return dialog;
+	return GTK_WINDOW(dialog);
 }
 
-static void
-gfu_main_release_install_response(GfuMain *self);
-
 static void
 gfu_main_release_install_response_cb(GtkDialog *dialog, int response_id, GfuMain *self)
 {
 	gtk_window_destroy(GTK_WINDOW(dialog));
+	self->dialog_current = NULL;
 
 	if (response_id != GTK_RESPONSE_OK) {
-		g_ptr_array_set_size(self->actions, 0);
+		g_ptr_array_set_size(self->dialogs, 0);
 		return;
 	}
 
-	g_ptr_array_remove(self->actions, dialog);
-	gfu_main_release_install_response(self);
+	g_ptr_array_remove(self->dialogs, dialog);
+	gfu_main_process_dialogs(self);
 }
 
 static void
-gfu_main_release_install_response(GfuMain *self)
+gfu_main_process_dialogs(GfuMain *self)
 {
-	GtkWidget *dialog;
-
-	if (self->actions->len == 0) {
-		gfu_main_release_install(self);
+	if (self->dialog_current != NULL)
+		return;
+	if (self->dialogs->len == 0) {
+		if (self->dialog_func != NULL) {
+			self->dialog_func(self);
+			self->dialog_func = NULL;
+		}
 		return;
 	}
-	dialog = GTK_WIDGET(g_ptr_array_index(self->actions, 0));
-	g_signal_connect(dialog,
+	self->dialog_current = GTK_WINDOW(g_ptr_array_index(self->dialogs, 0));
+	g_signal_connect(self->dialog_current,
 			 "response",
 			 G_CALLBACK(gfu_main_release_install_response_cb),
 			 self);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gtk_window_present(GTK_WINDOW(self->dialog_current));
 }
 
 static void
@@ -1264,14 +1326,17 @@ gfu_main_release_row_button_clicked_cb(GfuReleaseRow *row, GfuMain *self)
 
 	g_set_object(&self->release, release);
 
-	g_ptr_array_add(self->actions, gfu_main_release_show_confirmation(self));
+	gfu_main_add_dialog(self, gfu_main_release_show_confirmation(self), FALSE);
 	if (g_strcmp0(fwupd_device_get_branch(self->device),
 		      fwupd_release_get_branch(self->release)) != 0) {
-		g_ptr_array_add(self->actions, gfu_main_release_show_branch_warning(self));
+		gfu_main_add_dialog(self, gfu_main_release_show_branch_warning(self), FALSE);
 	}
 	if (fwupd_device_has_flag(self->device, FWUPD_DEVICE_FLAG_AFFECTS_FDE))
-		g_ptr_array_add(self->actions, gfu_main_release_show_fde_warning(self));
-	gfu_main_release_install_response(self);
+		gfu_main_add_dialog(self, gfu_main_release_show_fde_warning(self), FALSE);
+
+	/* when all the dialogs have been agreed to, install the release */
+	self->dialog_func = gfu_main_release_install_cb;
+	gfu_main_process_dialogs(self);
 }
 
 static void
@@ -1301,8 +1366,7 @@ gfu_main_device_verify_cb(GObject *source_object, GAsyncResult *res, gpointer us
 	    /* TRANSLATORS: firmware is cryptographically identical */
 	    _("%s firmware checksums matched"),
 	    fwupd_device_get_name(self->device));
-	g_signal_connect(dialog, "response", G_CALLBACK(gtk_window_destroy), NULL);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	gfu_main_refresh_device(self);
 }
 
@@ -1316,7 +1380,6 @@ gfu_main_device_verify_response_cb(GtkDialog *dialog_parent, int response_id, Gf
 					  gfu_main_device_verify_cb,
 					  self);
 	}
-	gtk_window_destroy(GTK_WINDOW(dialog_parent));
 }
 
 static void
@@ -1338,7 +1401,7 @@ gfu_main_device_verify_clicked_cb(GtkWidget *widget, GfuMain *self)
 	    /* TRANSLATORS: device will "go away" and then "come back" */
 	    _("The device may be unusable during this action"));
 	g_signal_connect(dialog, "response", G_CALLBACK(gfu_main_device_verify_response_cb), self);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 }
 
 static void
@@ -1365,7 +1428,6 @@ gfu_main_device_verify_update_response_cb(GtkDialog *dialog, int response_id, Gf
 						 gfu_main_verify_update_cb,
 						 self);
 	}
-	gtk_window_destroy(GTK_WINDOW(dialog));
 }
 
 static void
@@ -1390,7 +1452,7 @@ gfu_main_device_verify_update_cb(GtkWidget *widget, GfuMain *self)
 			 "response",
 			 G_CALLBACK(gfu_main_device_verify_update_response_cb),
 			 self);
-	gtk_window_present(GTK_WINDOW(dialog));
+	gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 }
 
 static void
@@ -1691,7 +1753,7 @@ gfu_main_unlock_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 				 "response",
 				 G_CALLBACK(gfu_main_reboot_shutdown_response_cb),
 				 self);
-		gtk_window_present(GTK_WINDOW(dialog));
+		gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	} else if (fwupd_device_has_flag(self->device, FWUPD_DEVICE_FLAG_NEEDS_REBOOT)) {
 		dialog =
 		    gtk_message_dialog_new(window,
@@ -1709,7 +1771,7 @@ gfu_main_unlock_cb(GObject *source, GAsyncResult *res, gpointer user_data)
 				 "response",
 				 G_CALLBACK(gfu_main_reboot_restart_response_cb),
 				 self);
-		gtk_window_present(GTK_WINDOW(dialog));
+		gfu_main_add_dialog(self, GTK_WINDOW(dialog), TRUE);
 	}
 }
 
@@ -1760,7 +1822,10 @@ gfu_main_client_connect_cb(GObject *source, GAsyncResult *res, gpointer user_dat
 	    self->client,
 	    FWUPD_FEATURE_FLAG_FDE_WARNING | FWUPD_FEATURE_FLAG_SWITCH_BRANCH |
 		FWUPD_FEATURE_FLAG_SHOW_PROBLEMS | FWUPD_FEATURE_FLAG_REQUESTS |
-		FWUPD_FEATURE_FLAG_UPDATE_ACTION,
+#if FWUPD_CHECK_VERSION(1, 9, 8)
+		FWUPD_FEATURE_FLAG_REQUESTS_NON_GENERIC |
+#endif
+		FWUPD_FEATURE_FLAG_ALLOW_AUTHENTICATION | FWUPD_FEATURE_FLAG_UPDATE_ACTION,
 	    self->cancellable,
 	    gfu_main_set_feature_flags_cb,
 	    self);
@@ -1875,8 +1940,8 @@ gfu_main_free(GfuMain *self)
 		g_object_unref(self->client);
 	if (self->application != NULL)
 		g_object_unref(self->application);
-	if (self->actions != NULL)
-		g_ptr_array_unref(self->actions);
+	if (self->dialogs != NULL)
+		g_ptr_array_unref(self->dialogs);
 	if (self->post_requests != NULL)
 		g_ptr_array_unref(self->post_requests);
 	g_free(self);
@@ -1941,7 +2006,7 @@ main(int argc, char **argv)
 			 G_CALLBACK(gfu_main_device_request_cb),
 			 self);
 
-	self->actions = g_ptr_array_new_with_free_func((GDestroyNotify)gtk_window_destroy);
+	self->dialogs = g_ptr_array_new_with_free_func((GDestroyNotify)gtk_window_destroy);
 	self->post_requests = g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 
 	/* ensure single instance */
